#pragma once

#ifdef __cplusplus
extern "C" {
#endif

#pragma pack(push,8)


namespace ProgDvbEngine
{
	namespace Dialogs
	{
		struct SBase;
	}
};

struct SCIMenu;
struct SCISatus;
struct SCIInputRequest;


namespace ProgModuleApi2
{
	const int	API_VERSION=208;
	const int	DEVICE_MAX_FILTER_COUNT=128;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////// Unit structures and interfaces
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	struct SUnitInfo
	{
		int		m_StructSize;					//in
		HWND	m_hWndMainInterfaceWindow;		//in
		wchar_t*m_pConfigFilename;				//in
		int		m_ApiVersion;					//in/out
		int		m_Version;						//out		
		int		m_ApiVersionRequest;			//out
		wchar_t	m_Author[200];					//out

		int					m_SubDeviceCount;			//out
		unsigned __int32	m_SubDeviceIDArray[100];	//out
		bool			m_bDisableExtEBDAModules;	// in,for BDA module only
	};

	struct EModuleCaps
	{
		enum EValue
		{
			FILTERS					=   0x0001,
			DISEQ_CMD				=   0x0004,
			REMOTE_CONTROL			=   0x0010,
			OPTIONS					=   0x0100,
			POWER_OFF_LNB			=   0x0200,
			CI						=   0x0800,
			BLIND_SCAN				=	0x1000,
			RF_SCAN					=	0x2000,
			RF_MIS_CONTROL			=	0x4000,
			NETWORK					=  0x80000,
			RAW_TS					= 0x200000,
			TONE_BUST				= 0x800000,
			NO_TUNER				=0x2000000,
			MAC						=0x4000000,
			GET_CHANNELS			=0x8000000,
		};
	};

	struct SModuleInfo
	{
		enum EStreamType
		{
			ST_TS=0,
			ST_PS,
			ST_WMV
		};

		int			m_StructSize;					//in
		DWORD		m_ID;							//in
		
		wchar_t		m_Name[100];					//out
		int			m_DialogID;						//out
		DWORD		m_dwCaps;						
		bool		m_bCanStart;					//out, Only Device
		int			m_MaxFilterCount;				//out, 0 for unlim,Only Device
		EStreamType	m_StreamType;					//out,Only Device
		
		unsigned char	m_MAC[6];						//out,Only Device
	};
	

	interface ICI;
	interface IEngineApi;
	struct SCreateModuleParams
	{
		int						m_StructSize;		//in
		unsigned __int32		m_ID;				//in
		IEngineApi*				m_pEngineApi;		//in
		ChListFormat::TDeviceID	m_DeviceID;			//in, Only Device, m_Type not filled !!!

		ChListFormat::ETunerType	m_TunerType;		//out; Only Device
		
		bool				m_bEnableCI;		//in;  Only Device
		ICI*				m_pCI;				//out; Only device

		SModuleInfo			m_ModuleInfo;		
	};
	
	interface IDeviceModule;
	interface IBaseApi2Module;
	interface IUnit
	{
		virtual bool	GetInfo(ProgModuleApi2::SUnitInfo*)=0;
		virtual bool	GetModuleInfo(ProgModuleApi2::SModuleInfo*mi)=0;

		virtual ProgModuleApi2::IBaseApi2Module*	CreateModule(ProgModuleApi2::SCreateModuleParams*)=0;
		virtual void								DestroyModule(ProgModuleApi2::IBaseApi2Module*)=0;

		virtual	bool  ShowOptionsDialog(DWORD subID,ProgModuleApi2::IBaseApi2Module*pmodule,HWND hParent)=0;					//pmodule = 0 for access to settings without start module
		virtual	bool  SetDialogSettings(DWORD subID,ProgModuleApi2::IBaseApi2Module*pmodule,ProgDvbEngine::Dialogs::SBase*)=0; //
		virtual	bool  GetDialogSettings(DWORD subID,ProgModuleApi2::IBaseApi2Module*pmodule,ProgDvbEngine::Dialogs::SBase*)=0; //

		virtual	const wchar_t* GetLastError()=0; // null if OK
	};

	typedef ProgModuleApi2::IUnit* PUnit;
	PUnit	__declspec(dllexport) __cdecl ProgModuleApi2_GetDeviceUnit(); // for device
	PUnit	__declspec(dllexport) __cdecl ProgModuleApi2_GetModuleUnit2(); // for module

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////// Device structures and interfaces
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	struct SEngineFilter
	{
		interface IReceiver
		{
			virtual void EngineFilter_ReceiveTraffic(byte*data,int size)=0;
		};
		enum	EType
		{
			FT_Packet184=0,
			FT_Packet920=1,
			FT_PSISection=2,
			FT_PESSection=3,
			FT_TS=4,
			FT_RawTS=5
		};

		int		m_StructSize;
		int		m_Pid;
		wchar_t	m_Name[100];
		
		EType	m_Type;
		byte	m_SectionData;
		byte	m_SectionMask;	
		IReceiver*	m_pReceiver;
	};
	
	struct SDeviceTunerState
	{
		enum ENetworkConnectionStatus
		{
			ncsOffline=0,            // No connection
			ncsLocating,            
			ncsConnecting,          
			ncsBuffering,           
			ncsPlaying,             
			ncsError=-1,                // Any error

			ncsNetError_HostNotFounded			=-102,
			ncsNetError_BadAddress				=-103,
			ncsNetError_DnsError				=-104,  
			ncsNetError_ConnectionFailed		=-105,
			ncsNetError_SendDataFailed			=-106,
			ncsNetError_InvalidURL				=-107,
			ncsNetError_AnswerNotRecognized		=-108,
			ncsNetError_ResourceNotFounded404	=-109,
			ncsNetError_HtmlAnswer				=-110,                                
			ncsNetError_TimeOut					=-111,
			ncsNetError_DllNotFounded			=-112,
			ncsNetError_NotSupported			=-113,
			ncsNetError_401InvalidLoginPass		=-114
		}; 

		enum EFieldPresent
		{
			EFP_NetworkStatus	=1, 
			EFP_TrafficCounter	=2,            
			EFP_bSignalPresent	=4,          
			EFP_bSignalLocked	=8,           
			EFP_LevelMDecibel	=16,             
			EFP_LevelPercent	=32,
			EFP_Quality			=64,
			EFP_Device			=128
		}; 

		int	   						m_StructSize;
		int							m_Fields;			//EFieldPresent

		bool						m_bSignalPresent;
		bool						m_bSignalLocked;
		int            				m_LevelPercent;
		int            				m_LevelMDecibel ; // decibel /1000;
		int            				m_Quality;
		
		__int64         			m_TrafficCounter; // traffic of current channel
		ENetworkConnectionStatus	m_NetworkStatus;

		ChListFormat::TDeviceID		m_DeviceID;	// Only for EngineApi_OnTunerStateChanged
	};


	
	struct SSetTransponderAdditionParams
	{
		int							m_StructSize;
		int							m_DelayMs;			//out, only for device
		SModuleInfo::EStreamType	m_StreamType;		//out, only for module
		
	};

	struct SLcdInfo
	{
		int									m_StructSize;
		ChListFormat::TChannelInfoStruct	m_Channel;	//structsize=0 if file
		wchar_t								m_PlayBackFileName[MAX_PATH];// ==0 if TV

		int			m_Signal_Level;
		int			m_Signal_Quility;
		bool		m_Signal_TrafficPressent;
		bool		m_Signal_TrafficCrypted;
		bool		m_Signal_TrafficKeyPressent;
		SDeviceTunerState::ENetworkConnectionStatus	m_NetworkStatus;

		bool		m_bIsRecording;
		bool		m_bIsPaused;

		FILETIME	m_Epg_UtcStartTime;
		FILETIME	m_Epg_Duration;
		wchar_t		m_Epg_Text[5000];

		int			m_Sound_VolumeLevel;
		bool		m_Sound_Mute;			
	};

	interface IEngineApi
	{
		virtual	void	EngineApi_TrafficReceiver(byte*data,int size)=0;

		virtual	void	EngineApi_OnTunerStateChanged(SDeviceTunerState*s)=0;
		virtual bool	EngineApi_GetCurrentDeviceInfo(ProgModuleApi2::SModuleInfo*)=0;
		virtual	int		EngineApi_OpenFilter(SEngineFilter*s)=0;
		virtual	void	EngineApi_CloseFilter(int)=0;

		virtual	void	EngineApi_PostRemoteControlKey(const char*key)=0;
		
		virtual	void	EngineApi_CI_SendMenu(SCIMenu*)=0;
		virtual	void	EngineApi_CI_SendStatus(SCISatus*)=0;
		virtual	void	EngineApi_CI_RequetInput(SCIInputRequest*)=0;

		virtual bool	EngineApi_ChList_SetChannel(ChListFormat::TChannelIdEx*id)=0;
		virtual bool	EngineApi_ChList_SendM3uList(const wchar_t*text)=0;
		virtual bool	EngineApi_ChList_SendCurrentUrl(const wchar_t*text)=0;
		virtual int		EngineApi_ChList_GetCurrentChannelList(ChListFormat::TChannelInfoStruct*channels)=0; //channels=0 for size detection
		virtual int		EngineApi_ChList_GetCurrentTransponder(ChListFormat::TChannelInfoStruct*channels)=0; //channels=0 for size detection
		virtual int		EngineApi_ChList_SendChannelList(ChListFormat::TChannelInfoStruct*channels)=0; 

		virtual bool	EngineApi_GetLcdInfo(ProgModuleApi2::SLcdInfo*)=0; // API version >=201

		virtual HWND	EngineApi_GetCommandWindowForChannel(int sid)=0; // API version >=203, sid=0 for current
		virtual bool	EngineApi_NotificationAboutUsingOfChannels(ChListFormat::TChannelInfoStruct*channels,int count)=0; // API version >=203, for open in CI
	};


	interface IBaseApi2Module
	{

	};

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////             Module                                                      //////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


	interface IModule : public IBaseApi2Module
	{
		virtual	void	SetTransponder(ChListFormat::TChannelInfoStruct*ch,ProgModuleApi2::SSetTransponderAdditionParams*)=0; //ch=0 for stop Channel		
		
		//all traffic from transponder
		virtual void	ReceiveTraffic(byte*data,int size)=0; 

		 // from command line
		virtual void	SendCommand(const wchar_t*)=0;

		// imporant if more one channel from one device (in TS)
		virtual void	UpdateOpenedChannels(ChListFormat::TChannelInfoStruct*channels,int count,SModuleInfo::EStreamType st)=0; 
	};
}



#pragma pack(pop)

}; //extern

